/*
 * Copyright (C) 1996-2001 LAAS/CNRS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
/* $LAAS: GDHE_packets.h,v 1.1 2005/10/27 12:54:33 matthieu Exp $ */

#ifndef _GDHE_PACKETS_H
#define _GDHE_PACKETS_H
/**********************************************************************
  file name/programme name: packets.h
  original date/person    : xx/xx/1992
  modified dates/persons  : 03/05/1994   add new request 'select_robot_type'
***********************************************************************
   definitions of constant and data structure to send a request to server
**********************************************************************/

#ifdef HAVE_GDHE_TERRAIN
#include "gdhe_terrain.h"
#endif

/***************************************************/
/*****    Values to send command from clients  *****/
/***************************************************/

/* We should have common coding for the commands */

#define PLACE_ROBOT         1
#define SET_PLATFORM        2
#define PLACE_DIAMOND       3
#define PLACE_DOORS         4
#define PLACE_WALLS         5
#define PLACE_WALLS_REL     6
#define PLACE_WALLS_ACC     7
#define PLACE_FREE_SPACE    8
#define PLACE_TRAJECTORY    9
#define PLACE_PATH         10
#define ERASE_ROBOT        11
#define ERASE_DIAMOND      12
#define ERASE_DOORS        13
#define ERASE_WALLS        14
#define ERASE_FREE_SPACE   15
#define ERASE_TRAJECTORY   16
#define ERASE_PATH         17
#define PLACE_GOAL_O       18
#define PLACE_GOAL_NO      19
#define ERASE_GOAL         20

#define EXEC_TRAJ          21
#define MOVE_PLATFORM      22

#define VIDEO_ON           23
#define VIDEO_OFF          24
#define LASER_ON           25
#define LASER_OFF          26

#define SELECT_ROBOT_TYPE  27
#define UPDATE_PER_OBST    28
#define ERASE_PER_OBST     29

#define PLACE_ROBOT_TRAJ   30
#define ERASE_ROBOT_TRAJ   31
#define PLACE_ROBOT_PATH   32
#define ERASE_ROBOT_PATH   33

#define PLACE_RS_TRAJ      34
#define ERASE_RS_TRAJ      35

#define PLACE_DEPENDENCY_EDGE  36
#define ERASE_DEPENDENCY_EDGE  37
#define PLACE_DEPENDENCY_CYCLE 38
#define ERASE_DEPENDENCY_CYCLE 39

#define PLACE_MISSION_GOAL  40
#define ERASE_MISSION_GOAL  41

#define EVAL_EXPRESSION_ADD 45
#define EVAL_EXPRESSION     42

#define PLACE_TERRAIN       43
#define ERASE_TERRAIN       44

#define DISCONNECT          0

#define CLOSE_ALL          -1

/* And we need a few macros */

#define MAX_CONNECTIONS    30
#define MAX_WALLS          12
#define MAX_RS_TRAJ        12

#define MAX_ROBOT          MAX_CONNECTIONS
#define MIN_ROBOT_ID        0
#define MAX_ROBOT_ID       ( MAX_ROBOT - 1 )

#define MISSION_GOAL_LENGTH 256
#define EVAL_PACKET_LENGTH  256
#define EVAL_BUFFER_LENGTH  32768

#define PACKET_DIVIDER     20 /* ancien MAX(MAX_ROBOT, MAX_WALLS) */

/* Maximal number of robot type registered in the server */

#define MAX_ROBOT_TYPE     12
#define MAX_TYPE_ID        (MAX_ROBOT_TYPE - 1)
#define MIN_TYPE_ID         0

/* First say what an angle is */

typedef float angle;

/* The basic argument is a location */

typedef struct location {
  float x, y;
} location;

/* Then we can use segments */

typedef struct segment {
  location* p1;
  location* p2;
} segment;

/* Finally, we also want a location with angle */

typedef struct position {
  location* p;
  float theta;
} position;

/* Then, we need vectors for both locations and segments */

typedef struct v_location {
  int number;
  location* array;
} v_location;

typedef struct v_position {
  int number;
  position* array;
} v_position;

typedef struct v_segment {
  int number;
  segment* array;
} v_segment;


typedef struct trajectory {
  angle theta0;
  angle theta1;
  v_location* vloc;
} trajectory;

/* A Reed & Sheep trajectory segment */
typedef struct rs_segment {
    angle dt;
    float dl;
} rs_segment;

typedef struct rs_trajectory {
    position *initpos;
    int number;
    rs_segment *array;
} rs_trajectory;

typedef struct polygon {
  int number;
  v_location* polys;
  int* boolean;
} polygon;


/* a cycle in the dependency graph */
typedef struct dependency_cycle {
    int number;
    int *nodes;
} dependency_cycle;


/* All arguments need be formatted: here comes the packet structure */

typedef struct packet {
    int which;
    int command;		/* Command is used both to understand 
				 *  the will of the driver and as tag */ 
    union {
	angle        *ang;
	location     *loc;
	position     *pos;
	segment      *seg;
	v_location   *vloc;
	v_position   *vpos;
	v_segment    *vseg;
	polygon      *pol;
	trajectory   *traj;
	rs_trajectory *rs_traj;
	int          *robot_type;   /*  argument for 'select_robot_type'  */
	int          *edge_to;
	dependency_cycle *cycle;
	char *str;
#ifdef HAVE_GDHE_TERRAIN
	GDHE_MNT *terrain;
#endif
    } args;
} packet;

#endif /* _GDHE_PACKETS_H */
