# 
# Copyright (C) 1996-2001 LAAS/CNRS 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# $LAAS: lutTerrain.tcl,v 1.5 2007/01/17 08:28:04 simon Exp $

# Note: "MNT" is the french acronym for Digital Terrain Model ("Modèle
# Numérique de Terrain")

# Various functions to build a color look up table (lut) for the terrain
# display. ::mntColor contains the RGB values to display, indexed by a char
# (0-255) stored in the "type" field of the GDHE_MNT_POINT structure.

proc mntLutBW {} {
    # Black and white lut
    for {set i 0} {$i < 256} {incr i} {
	set ::mntColor($i) "$i $i $i"
    }
}

proc mntLutGreen {} {
    # Uniform green lut
    for {set i 0} {$i < 256} {incr i} {
	set ::mntColor($i) { 60 120 0 }
    }
}

proc mntLutGold {} {
    # Uniform goldish lut
    for {set i 0} {$i < 256} {incr i} {
	set ::mntColor($i) { 200 160 40 }
    }
}

proc mntLutRainbow {} {
    # Rainbow lut
    for {set i 0} {$i < 128} {incr i} {
	set ::mntColor($i) "[expr 255 - (2 * $i)] [expr 2 * $i] 0"
    }
    for {set i 128} {$i < 256} {incr i} {
	set ::mntColor($i) "0  [expr 255 - (2 * ($i - 128))] [expr 2 * $i]"
    }
}
