#
# Copyright (c) 2006 CNRS/LAAS
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# Matthieu Herrb - Thu Feb  2 2006
#
# $LAAS: martha.tcl,v 1.1 2006/02/02 16:05:48 matthieu Exp $
#
###
### Procedures de dessin de l'environnement a` la mode Martha
###
### This code used to be in env.tcl, and used to be called from draw_all
### 
### It's kept there for reference only
###
#----------------------------------------------------------------------
## 
## Stations 
##
proc draw_stations {} {
    global station_vertices station_boundaries

    set w .gdhe.ogl

    color 0 0 200
    foreach {station} [array names station_boundaries] {
	set poly {} 
	foreach {vertex} [lrange $station_boundaries($station) 1 end] {
	    set poly [concat $poly $station_vertices($vertex)]
	}
	set cmd [concat polygon \
		  [lindex $station_boundaries($station) 0] $poly]
	eval $cmd
    }
}

#----------------------------------------------------------------------
##
## Cellules
##
proc draw_cells {} {
    global cell_vertices cell_boundaries
    
    set w .gdhe.ogl

    foreach {cell} [array names cell_boundaries] {
	set poly {} 
	set line {} 
	foreach {vertex} [lrange $cell_boundaries($cell) 1 end] {
	    set poly [concat $poly $cell_vertices($vertex)]
	    set line [concat $line $cell_vertices($vertex) 0.01]
	}
	set cmd [concat polygon \
		  [lindex $cell_boundaries($cell) 0] $poly]
	color 60 80 80
	eval $cmd
	set cmd [concat polyline \
		     [lindex $cell_boundaries($cell) 0] $line]
	color 100 0 0
	eval $cmd
    }
}

#----------------------------------------------------------------------
##
## obstacles
##
proc draw_obstacles {} {
    global obstacle_vertices obstacle_boundaries
    
    set w .gdhe.ogl

    color 200 200 30
    cullFace True
    foreach {obstacle} [array names obstacle_boundaries] {
	set poly {} 
	foreach {vertex} [lrange $obstacle_boundaries($obstacle) 1 end] {
	    set poly [concat $poly $obstacle_vertices($vertex) -0.01]
	}
	set cmd [concat prism \
		  [lindex $obstacle_boundaries($obstacle) 0] 0 0 0.4 \
		     $poly ]
	# puts $cmd
	eval $cmd
    }
}

#----------------------------------------------------------------------
###
### draw_env - dessin de l'environnement
###
proc draw_env {} {
    global envDefined
    
    # est-ce qu'un environnement est defini ?
    
    if {[info exists envDefined]} {
	concave True
	object Environnement {
	    color 200 100 0
	    cullFace False
	    draw_cells
	    pushMatrix
	    translate 0 0 0.01
	    draw_stations
	    popMatrix
	    draw_obstacles
	}
	draw_zone_stockage
	concave False
    }
}

#----------------------------------------------------------------------
global wallColor
set wallColor(0) { 130 20 20 }
set wallColor(1) { 130 60 20 }
set wallColor(2) { 130 130 20 }
set wallColor(3) { 60 130 20 }
set wallColor(4) { 20 130 20 }
set wallColor(5) { 20 130 60 }
set wallColor(6) { 20 130 130 }
set wallColor(7) { 20 60 130 }
set wallColor(8) { 20 20 130 }
set wallColor(9) { 60 20 130 }
set wallColor(10) { 130 20 130 }
set wallColor(11) { 130 20 60 }


###
### Dessin des "Walls" d'un robot
###
proc draw_walls robot {
    global walls wallColor

    # hauteur des murs
    set wallHeight 1.0
    cullFace true
    concave false
    
    for { set i 0 } { $i < 12 } { incr i } {
	if {![info exists walls($robot,$i)]} {
	    continue
	}
	# puts "wallColor: $wallColor($i)"
	eval [concat color $wallColor($i)]

	foreach  w $walls($robot,$i) {

	    set x0 [lindex $w 0]
	    set y0 [lindex $w 1]
	    set x1 [lindex $w 2]
	    set y1 [lindex $w 3]
	
	    polygon 4 $x0 $y0 0 $x1 $y1 0  \
		$x1 $y1 $wallHeight $x0 $y0 $wallHeight
	    polygon 4 $x1 $y1 0 $x0 $y0 0  \
		$x0 $y0 $wallHeight $x1 $y1 $wallHeight
	}
    }
}

#----------------------------------------------------------------------

###
### Dessin des obstacles de PER
###
proc draw_per_obst {} {
    global perObst
    
    concave True
    cullFace True
    color 250 50 50

    if {[info exists perObst]} {
	foreach o $perObst {
	    set m [expr [llength $o]/2]
	    set c "prism $m 0 0 1"
	    for {set i 0} {$i < $m} {incr i} {
		lappend c [lindex $o [expr 2*$i]] [lindex $o [expr 2*$i+1]] 0.0
	    }
	    eval $c
	}
    }
}

proc draw_trajectory robot {
    global trajectory wallColor

    pushMatrix
    translate 0 0 .1
    
    if {[info exists trajectory($robot)]} {
	# puts $trajectory($robot)
	color 255 255 255
	eval polyline "$trajectory($robot)"
    }
    popMatrix
}

#---------------------------------------------------------------------
#-#
## Mise a jour du enviroment des containers.
##

# Put down un container dans la zone de stockage
proc pd_container { zone_stockage container } {
    global stockage_existance

    set stockage_existance($zone_stockage) [concat $stockage_existance($zone_stockage) $container]
}

# Pick up un container dans la zone de stockage
proc pu_container { zone_stockage container } {
    global stockage_existance

    set stockage_existance($zone_stockage) [concat [lrange $stockage_existance($zone_stockage) 0 1]\
                           [lreplace [lrange $stockage_existance($zone_stockage) 2 end] \
                           [lsearch [lrange $stockage_existance($zone_stockage) 2 end ] $container] \
                           [lsearch [lrange $stockage_existance($zone_stockage) 2 end ] $container] ] ]
}

#----------------------------------------------------------------------
##
## Zone Stockage
##
proc draw_zone_stockage {} {
    global stockage_existance stockage_position stockage_angle enviromentType

    set w .gdhe.ogl

    pushMatrix
    color 200 0 0
    cullFace True
    foreach {zone} [array names stockage_position] {
	pushMatrix
	translate [lindex $stockage_position($zone) 0] [lindex $stockage_position($zone) 1] 0
	rotate [lindex $stockage_angle($zone) 0] 0 0 1
        base_stockage_$enviromentType
	for { set i 1 } {$i <= [expr [lindex $stockage_existance($zone) 1] - 1]} { incr i 1 } { 	    
	    eval module_container_$enviromentType
	}
	popMatrix
	pushMatrix
	draw_stockage $zone
	popMatrix
    }
    popMatrix
}
	
#----------------------------------------------------------------------
##
##  Stockage
##
proc draw_stockage { zone } {
    global stockage_existance 

    for {set i 2} {$i<[llength $stockage_existance($zone)]} {incr i 1} {
	if {[lindex $stockage_existance($zone) 1] >= [lindex $stockage_existance($zone) $i]} {
	    eval draw_container $zone [lindex $stockage_existance($zone) $i]
	} else {
	    puts -nonewline "Error of containers number in stockage_existance in "
	    puts $i
	}
    }
}
	
	
#----------------------------------------------------------------------
##
## Containers
##
proc draw_container { zone numero } {
    global stockage_position stockage_existance stockage_angle enviromentType
    
    if {$numero<=[lindex $stockage_existance($zone) 1]} {     
	pushMatrix
	cullFace True

	translate [lindex $stockage_position($zone) 0] [lindex $stockage_position($zone) 1] 0
	rotate [lindex $stockage_angle($zone) 0] 0 0 1
	
	if {![string compare $enviromentType hilare ]} {
	    translate 0 0 -0.17
	}
	set depl 0
	for { set i 1 } {$i <= $numero} {incr i 1} {
	    if {![string compare $enviromentType hilare ]} {
		set depl [expr $depl+0.13]	   
	    }
	    if {![string compare $enviromentType martha ]} {
		set depl [expr $depl+3.1]	
	    }	    
	}
        if {![string compare $enviromentType hilare ]} {
	    translate 0.04 0 $depl                               ;# Correction manuel
	}
        if {![string compare $enviromentType martha ]} {
	    translate 0.00 0 $depl                            
	}
	container_$enviromentType
	popMatrix
    } else {
	puts -nonewline  "Error When draw container because of the position "
	puts $zone
    }
}
    
       


