# 
# Copyright (C) 1996-2001 LAAS/CNRS 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# $LAAS: terrain.tcl,v 1.11 2007/01/17 08:45:13 matthieu Exp $

package require Gdhe::terrain

source $gdhe_datadir/examples/lutTerrain.tcl

# Note: "MNT" is the french acronym for Digital Terrain Model ("Modle
# Numrique de Terrain")

# A simple color look-up table
set mntColor(0) { 120 60 0 }
set mntColor(1) { 60 120 0 }

# This is the function to call to display a MNT stored in a file 
# 
# - fileName: the MNT file (e.g. the file ./examples/t3 in the gdhe source
# hierarchy, installed in share/gdhe/images after a build)
# - color selects the color LUT to use
# - cropDisplay: when set to 1, only displays the cells within the rectangle
# that bounds the ones for  which an elevation is set in the model (i.e. the
# GDHE_MNT_POINT for which z != 0 && type != 0). Recommanded.
# - lightDisplay: if set to '0', the cells for wich no elevation is set in the
# model are displayed at elevation zero, which does not look very nice,
# depending on the elevations of the other cells. If set to 1, only the set
# cells are displayed (note that because of the way GDHE and opengl handles
# terrain displays with facets, a single unset cell will also cause its nearest 
# neighbors to be withdrawned from the display).

proc initMntDisplay {fileName color cropDisplay lightDisplay} {
    if {$color != 0} {
	mntLutBW;
    } else {
	mntLutGreen
    }
    set ::mntColor(0) { 120 60 0 };	# So that the ground is always brown

    readTerrain terrainStruct $fileName
    set ::robots(pipo) " placeMnt terrainStruct $cropDisplay $lightDisplay "
    set ::pos(pipo) { 0 0 0 }
    puts "Initializing the DEM display";
    redrawAllWindows;
}

# Reads a MNT file and provide a GDHE object
proc placeMnt {mnt cropDisplay lightDisplay} {
    
    object $mnt {
	cullFace false
	terrain $mnt $cropDisplay $lightDisplay
    }
}

# This is to be called when a terrain model is incrementally updated (typically
# as a rover moves and maps newly discovered areas), by reading "MNT chunks"
# files. 
# Example:
# initMntDisplay share/gdhe/images/dem.Init 0 1 1 
# updateMnt terrainStruct share/gdhe/images/dem.0
# updateMnt terrainStruct share/gdhe/images/dem.5
# updateMnt terrainStruct share/gdhe/images/terrains/dem.10 ... dem.50
proc updateMnt {mnt fileName} {
    updateTerrain $mnt $fileName
    objectDelete $mnt
    puts "Updating the DEM display";
    redrawAllWindows
}

# Command sequence to remove a gdhe MNT object:
# unset robots(mnt)
# deleteTerrain dem0
# objectDelete dem0

readTerrain dem0 $gdhe_datadir/images/t3
set robots(mnt) { placeMnt dem0 0 1 }
set pos(mnt) { -10 -10 0 }

# Define obervator's position
.gdhe.ogl.pos.dist set 20
.gdhe.ogl.pos.elev set 25
.gdhe.ogl.pos.x set 5
.gdhe.ogl.pos.y set 5 

