/*
 * Copyright (C) 1996-2003 LAAS/CNRS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "config.h"
__RCSID("$LAAS: light.c,v 1.6 2004/08/25 13:50:01 matthieu Exp $");

#include <tcl.h>

#include "togl.h"
#include "global.h"
#include "light.h"

/**
 ** Fonction de trace des variables grant l'clairage
 **/
char *
setLightCallback(ClientData clientData, Tcl_Interp *interp,
		 CONST84 char *name1, CONST84 char *name2, int flags)
{
    Tcl_Obj *arg;

    printf("setLights %s(%s)\n", name1, name2);

    if (flags & TCL_TRACE_WRITES) {
	arg = Tcl_NewStringObj("setLights", -1);
	
	setLightsCallBack(clientData, interp, 1, &arg);
    }
    return NULL;
}

/*----------------------------------------------------------------------*/

int
setLightsCallBack(ClientData clientData, Tcl_Interp *interp, 
		  int argc, Tcl_Obj *const argv[])
{
    Tcl_Obj *value, **largv;
    int i, j;
    char buf[16];
    int largc;
    GLfloat v[4];
    double tmp;


    if (argc != 1) {
	Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \""
						  "setLights \"",
						  -1));
	return TCL_ERROR;
    }
    /* parametres d'eclairage */
    for (i = 0; i < 8; i++) {
	snprintf(buf, sizeof(buf), "LIGHT%d", i);
	/* position */
	value = Tcl_ObjGetVar2(interp, 
			       Tcl_NewStringObj("::Gdhe::lights::position", -1), 
			       Tcl_NewStringObj(buf, -1), TCL_GLOBAL_ONLY);
	if (value != NULL) {
	    if (Tcl_ListObjGetElements(interp, value, &largc, &largv)
		== TCL_ERROR) {
		Tcl_BackgroundError(interp);
	    }
	    if (largc != 3 && largc != 4) {
		Tcl_SetObjResult(interp, 
				 Tcl_NewStringObj("setLight: error: "
						  "wrong number of elements",
						  -1));
		Tcl_BackgroundError(interp);
	    }
	    for (j = 0; j < 3; j++) {
		Tcl_GetDoubleFromObj(interp, largv[j], &tmp);
		light_position[i][j] = (GLfloat)tmp;
	    }
	    if (largc == 4) {
		Tcl_GetDoubleFromObj(interp, largv[3], &tmp);
		light_position[i][3] = (GLfloat)tmp;
	    } else {
		light_position[i][3] = 0.0;
	    }
	    
	    
	}
	glLightfv(GL_LIGHT0 + i, GL_POSITION, &light_position[i][0]); 

	/* ambient */
	value = Tcl_ObjGetVar2(interp, 
			       Tcl_NewStringObj("::Gdhe::lights::ambient", -1), 
			       Tcl_NewStringObj(buf, -1), TCL_GLOBAL_ONLY);
	if (value != NULL) {
	    if (Tcl_ListObjGetElements(interp, value, &largc, &largv)
		== TCL_ERROR) {
		Tcl_BackgroundError(interp);
	    }
	    if (largc != 3) {
		Tcl_SetObjResult(interp, 
				 Tcl_NewStringObj("setLight: error: "
						  "wrong number of elements",
						  -1));
		Tcl_BackgroundError(interp);
	    }
	    for (j = 0; j < 3; j++) {
		Tcl_GetDoubleFromObj(interp, largv[j], &tmp);
		v[j] = (GLfloat)tmp;
	    }
	    v[3]  = 1;
	    glLightfv(GL_LIGHT0 + i, GL_AMBIENT, v);
	    
	}
	/* diffuse */
	value = Tcl_ObjGetVar2(interp, 
			       Tcl_NewStringObj("::Gdhe::lights::diffuse", -1),
			       Tcl_NewStringObj(buf, -1), TCL_GLOBAL_ONLY);
	if (value != NULL) {
	    if (Tcl_ListObjGetElements(interp, value, &largc, &largv)
		== TCL_ERROR) {
		Tcl_BackgroundError(interp);
	    }
	    if (largc != 3) {
		Tcl_SetObjResult(interp, 
				 Tcl_NewStringObj("setLight: error: "
						  "wrong number of elements", 
						  -1));
		Tcl_BackgroundError(interp);
	    }
	    for (j = 0; j < 3; j++) {
		Tcl_GetDoubleFromObj(interp, largv[j], &tmp);
		v[j] = (GLfloat)tmp;
	    }
	    v[3] = 1;
	    glLightfv(GL_LIGHT0 + i, GL_DIFFUSE, v);
	    glLightfv(GL_LIGHT0 + i, GL_SPECULAR, v);

	}
	/* enabled */
	value = Tcl_ObjGetVar2(interp, 
			       Tcl_NewStringObj("::Gdhe::lights::enabled", -1), 
			       Tcl_NewStringObj(buf, -1), TCL_GLOBAL_ONLY);
	if (value != NULL) {
	    if (Tcl_GetBooleanFromObj(interp, value, &largc) == TCL_ERROR) {
		Tcl_BackgroundError(interp);
	    }
	    if (largc) {
		light_enabled[i] = 1;
	    } else {
		light_enabled[i] = 0;
	    }
	} 
	if (light_enabled[i]) {
	    glEnable(GL_LIGHT0 + i);
	} /* sebachange */ else {
	  /* glDisable(GL_LIGHT0 + i); */
	}
    } /* for */

    /* Let result string empty */
    Tcl_SetObjResult(interp, nullObj);
    return TCL_OK;
}

/*----------------------------------------------------------------------*/

/**
 ** interface de 'enableLightingNow'
 **/

int
enableLightingNowCallBack(ClientData clientData, Tcl_Interp *interp, 
	    int argc, Tcl_Obj *const argv[])
{

    if (argc != 1) {
	Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \""
						  "enableLightingNow \"",
						  -1));
	return TCL_ERROR;
    } 
    glEnable(GL_LIGHTING);
    /* Let result string empty */
    Tcl_SetObjResult(interp, nullObj);
    return TCL_OK;
}
/*----------------------------------------------------------------------*/
/**
 ** interface de 'disableLightingNow'
 **/
int
disableLightingNowCallBack(ClientData clientData, Tcl_Interp *interp, 
	    int argc, Tcl_Obj *const argv[])
{

    if (argc != 1) {
	Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \""
						  "disableLightingNow \"",
						  -1));
	return TCL_ERROR;
    } 
    glDisable(GL_LIGHTING);
    /* Let result string empty */
    Tcl_SetObjResult(interp, nullObj);
    return TCL_OK;
}
/*----------------------------------------------------------------------*/
/**
 ** interface de 'enableLighting'
 **/
int
enableLightingCallBack(ClientData clientData, Tcl_Interp *interp, 
	    int argc, Tcl_Obj *const argv[])
{

    if (argc != 2) {
	Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \""
						  "enableLighting \"",
						  -1));
	return TCL_ERROR;
    }
    if (Tcl_GetBooleanFromObj(interp, argv[1], &enable_lighting) != TCL_OK) {
	return TCL_ERROR;
    }
    /* Let result string empty */
    Tcl_SetObjResult(interp, nullObj);
    return TCL_OK;
}
