# 
# Copyright (C) 1996-2003 LAAS/CNRS 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# $LAAS: lama.tcl,v 1.15 2003/06/25 16:13:07 matthieu Exp $
#----------------------------------------------------------------------
###
### Modele du robot lama
###
package provide Gdhe::models 1.0

# variable qui indique s'il faut dessiner les reperes
global lamaReperes
if ![info exists lamaReperes] {
    set lamaReperes false
}

# Dessin conditionnel de repere
#     l: longueur des axes
# x y z: translation / repere courant
# r g b: couleur
#
proc repereCond {l x y z r g b} {
    global lamaReperes

    if {$lamaReperes} {
	pushMatrix
	translate $x $y $z
	color $r $g $b
	repere $l
	popMatrix
    }
}

# Une roue (avec couleur si $args == { r g b })
proc lama_roue { args } {
    if {[llength $args] == 3} {
	set r [lindex $args 0]
	set g [lindex $args 1]
	set b [lindex $args 2]
    } else {
	set r 140
	set g 140
	set b 130
    }

    color $r $g $b
    cylinder 0 .075 0 y .25 -.45 .375
    cylinder 0 .45 0 y -.45 0.1
    cylinder 0 .55 0 y -.45 .05 0.05
}

# l'un des essieux avant ou arriere
proc lama_essieu_mobile {a1 a2 a3 rack} { 
    
    pushMatrix
    #  origine au niveau de l'essieu central 
    translate 0 0 .225
    # angles alpha
    rotate $a3 1 0 0 
    rotate [expr $a1 - 24] 0 1 0
    
    object LamaEssieu1 {
	# le bras 
	color 10 10 10
	cylinder 0 0 0 x .1 .55
	translate .6 0 -.1
	# boite de connexion
	rotate 24 0 1 0    
	color 130 130 130
	box 0 0 0 .22 .14 .13
	translate .12 0 .065 
	# moteur peristaltisme
	cylinder 0. -.075 0 y 0.15 .15
	cylinder 0. -.09 0 y 0.13 .30
    }
    # Rack VME sur essieu arriere 
    if { $rack } {
	color 100 100 90
	box -.05 0 .075 .5 .55 .4 
	color 120 120 100
	box -.15 0.05 0.475 .3 .35 .08 
	color 160 160 130
	box -.05 -.20 .475 .03 0.07 0.1
	color 30 30 30
	cylinder -.05 -.22 .575 z 0.005 0.12
    }
    # rotation du moteur de peristaltisme (alpha)
    rotate [expr $a2 + 180] 0 1 0

    object LamaEssieu2 {
	color 10 10 10
	# breas de soutien des roues
	cylinder 0 0 0.06 z 0.1 .17
	cylinder 0 -.075 .220 y 0.1 .15
	# les 2 roues 
	translate 0 0 .225
	lama_roue 
	rotate 180 0 0 1
	lama_roue
    }
    popMatrix
}

# l'essieu central et tout ce qu'il y a dessus
proc lama_essieu_central {} {
    global objLamaEssieu3
    
    pushMatrix
    # repere
    repereCond 1.0 0 0 0 0 200 0
    object LamaEssieu3 {

	translate 0 0 .225
	lama_roue
	color 10 10 10
	cylinder 0 -.075 0 y 0.1 .15
	cylinder 0 0 0 z 0.1 .20
	color 150 150 150
	box 0 0 .20 .20 .15 .13
	# boite bleue
	color 30 100 90 
	box 0 0 .33 .25 .5 .25
	# le mat
	color 100 100 100 
	box -0.14 -0.075 .33 0.02 0.03 1.0
	box -0.14  0.075 .33 0.02 0.03 1.0
	box -0.14  0.00  1.33 0.02 0.18 0.03
	color 30 30 30 
	box -0.17 0.00 0.75 0.06 0.15 0.08
	rotate 180 0 0 1
	lama_roue
    }
    popMatrix
}

#----------------------------------------------------------------------
### 
### platine
###
proc lamaPlatine {azi site} {
    pushMatrix
    object lamaPlatineInf {
	translate -0.14 0 1.58
	color 30 30 30
	box -0.02 0.0 0 0.08 0.05 0.05
	color 180 180 180
	cylinder -0.075 0 0.025 x 0.012 0.02
    }
    # repere 
    repereCond 0.3 0 0 0.075 0 0 230
    rotate $azi 0 0 1
    object lamaPlatineMid {
	color 30 30 30
	box -0.02 0 0.051 0.08 0.05 0.05
	color 180 180 180
	cylinder -0.075 0 0.075 x 0.012 0.02
	translate 0 0 0.075
    }
    rotate $site 0 -1 0
    object lamaPlatineSup {
	color 30 30 30
	box 0 -0.028 -0.01 0.02 0.005 0.05
	box 0  0.028 -0.01 0.02 0.005 0.05
	color 100 100 100
	box 0 0 0.04 0.02 0.6 0.01
	color 20 20 20
	box -0.013 0.24 -0.005 0.053 0.045 0.045
	color 70 70 70
	cylinder 0.01 0.24 0.015 x 0.028 0.01
	cylinder 0.02 0.24 0.015 x 0.028 0.05 0.02
	cylinder 0.04 0.24 0.015 x 0.05 0.008
	color 20 20 20
	box -0.02 -0.24 -0.005 0.053 0.045 0.045
	color 70 70 70
	cylinder 0.01 -0.24 0.015 x 0.028 0.01
	cylinder 0.02 -0.24 0.015 x 0.028 0.05 0.02
	cylinder 0.04 -0.24 0.015 x 0.05 0.008
    }
    # reperes
    repereCond 0.2 0 0 0 0 230 230
    repereCond 0.2 0 0.24 0.015 0 230 230
    repereCond 0.2 0 -0.24 0.015 0 230 230
    popMatrix
}

#----------------------------------------------------------------------
##
## procedures principales
##
proc lama {alpha1 alpha2 beta1 beta2 beta3} {
    
    # changement de repere de beta3
    set beta3 [expr $beta3/2 - 40]

    #  origine au niveau de l'essieu central 
    translate 0 0 -.225
    
    lama_essieu_mobile $beta3 $beta1 $alpha1 False
    lama_essieu_central
    pushMatrix
    rotate 180 0 0 1
    lama_essieu_mobile $beta3 $beta2 [expr -$alpha2] True
    popMatrix
}

# l'essieu central tout seul (avec des couleurs flashy si on veut)
proc lama_essieu_central_seul { r g b } {
    pushMatrix
    # repere
    repereCond 1.0 0 0 0 0 200 0
    object LamaEssieuCentralSeul$r$g$b {
	translate 0 0 .225
	lama_roue $r $g $b
	color 130 130 140
	cylinder 0 -.075 0 y 0.1 .15
	rotate 180 0 0 1
	lama_roue $r $g $b
    }
    popMatrix
}

# Juste un repre pour montrer la position de lama
proc lama_repere_seul { r g b } {
    pushMatrix
    # repere
    repereCond 1.0 0 0 0 0 200 0
    object LamaRepereSeul$r$g$b {
	color $r $g $b
	repere 0.5
    }
    popMatrix
}

#----------------------------------------------------------------------
##
## fenetre de controle de la configuration du robot
##
proc lamaControls {} {
    toplevel .lama
    wm title .lama "lama"
    wm geometry .lama -0+500
    # axes
    scale  .lama.alpha1  -length 200  -label {alpha1} -from -35 -to 35 \
	-resolution 0.1 -command {setLama alpha1 } -orient horizontal
    scale  .lama.alpha2  -length 200  -label {alpha2} -from -35 -to 35 \
	-resolution 0.1 -command {setLama alpha2 } -orient horizontal
    scale  .lama.beta1  -length 200  -label {beta1} -from -45 -to 45 \
	-resolution 0.1 -command {setLama beta1 } -orient horizontal
    scale  .lama.beta2  -length 200  -label {beta2} -from -45 -to 45 \
	-resolution 0.1 -command {setLama beta2 } -orient horizontal
    scale  .lama.beta3  -length 200  -label {beta3} -from 45 -to 120 \
	-resolution 0.1 -command {setLama beta3 } -orient horizontal
    .lama.beta3 set 80

    scale .lama.theta -length 200 -label theta -from -180 -to 180 \
	-resolution 0.1 -command {setLamaPlatine theta} -orient horizontal 
    scale .lama.phi -length 200 -label phi -from -90 -to 90 \
	-resolution 0.1 -command {setLamaPlatine phi} -orient horizontal 

    pack .lama.alpha1 -fill x
    pack .lama.alpha2 -fill x
    pack .lama.beta1 -fill x
    pack .lama.beta2 -fill x
    pack .lama.beta3 -fill x
    pack .lama.theta -fill x
    pack .lama.phi -fill x
}

# Handler des evenements  sur les sliders
proc setLama { axis value } {
    global robots

    set robots(lama) { lama [.lama.alpha1 get] \
			 [.lama.alpha2 get] \
			 [.lama.beta1 get] \
			 [.lama.beta2 get] \
			 [.lama.beta3 get] }
    .gdhe.ogl redraw
}

proc setLamaPlatine { axis value } {
    global platform

    set platform(lama) { lamaPlatine [.lama.theta  get] [.lama.phi get] }

    .gdhe.ogl redraw
}
