#
# Copyright (c) 2006 LAAS/CNRS 
#
# Thomas Lemaire - Fri Jan 6 2006
#
# $Id: video.tcl,v 1.6 2007/02/07 15:51:32 matthieu Exp $
#

package provide Gdhe::video 1.0

namespace eval Gdhe::video {

    variable convertBin "convert"
    variable mencoderBin "mencoder"
    variable mplayerBin "mplayer"

    variable imagePrefix "gdhe."
    variable imageDir "images"
    variable imageFormat "ppm"
    variable imageIndex 0
    variable imageNbDigits "4"
    variable imageTmp "/tmp/gdhe"

    variable videoOutput "gdhe.avi"
    variable videoCodec "mpeg4"
    variable videoFps "15"
    variable videoBitrate "2000"

    variable gdheOgl ".gdhe.ogl"

    variable taskId
    
    variable coord "dist elev azi x y z"

    variable poseFrom 
    array set poseFrom {
	dist 0.0
	elev 0.0
	azi  0.0
	x    0.0
	y    0.0
	z    0.0
    }

    variable poseTo
    array set poseTo {
	dist 0.0
	elev 0.0
	azi  0.0
	x    0.0
	y    0.0
	z    0.0
    }

    variable poseDelta

    variable poseDeltaSign
    array set poseDeltaSign {
	dist 1
	elev 1
	azi  1
	x    1
	y    1
	z    1
    }

    variable poseCur
    array set poseCur {
    	dist 0.0
	elev 0.0
	azi  0.0
	x    0.0
	y    0.0
	z    0.0
    }

    proc setDefaultPoseDelta {} {
	variable poseDelta
	array set poseDelta {
	    dist 1.0
	    elev 2.0
	    azi  2.0
	    x    0.1
	    y    0.1
	    z    0.1
	}
    }

    proc init {} {
	variable coord

	setDefaultPoseDelta

	set w ".gdheVideo"
	toplevel $w
	wm title $w "Video"
	wm resizable $w 0 0

	#### camera motion ####

	set fm "$w.motion"
	labelframe $fm -padx 2 -pady 2 -text "camera motion"

	set fmp "$fm.poses"
	frame $fmp

	# width of the coordinate entries
	set ew 5

	label $fmp.emptyLabel
	foreach c $coord {
	    label $fmp.${c}Label -text $c
	}

	grid $fmp.emptyLabel $fmp.distLabel $fmp.elevLabel $fmp.aziLabel $fmp.xLabel $fmp.yLabel $fmp.zLabel

	label $fmp.fromLabel -justify "right" -text "from:"
	foreach c $coord {
	    entry $fmp.from${c} -width $ew -textvariable ::Gdhe::video::poseFrom($c)
	}

	grid $fmp.fromLabel $fmp.fromdist $fmp.fromelev $fmp.fromazi $fmp.fromx $fmp.fromy $fmp.fromz

	label $fmp.toLabel -justify "right" -text "to:"
	foreach c $coord {
	    entry $fmp.to${c} -width $ew -textvariable ::Gdhe::video::poseTo($c)
	}

	grid $fmp.toLabel $fmp.todist $fmp.toelev $fmp.toazi $fmp.tox $fmp.toy $fmp.toz

	label $fmp.deltaLabel -justify "right" -text "delta:"
	foreach c $coord {
	    entry $fmp.delta${c} -width $ew -textvariable ::Gdhe::video::poseDelta($c)
	}

	grid $fmp.deltaLabel $fmp.deltadist $fmp.deltaelev $fmp.deltaazi $fmp.deltax $fmp.deltay $fmp.deltaz

	# 	label $fmp.signLabel -justify "right" -text "sign:"
	# 	foreach c $coord {
	# 	    entry $fmp.sign${c} -width $ew -state "disable"  -textvariable ::Gdhe::video::poseDeltaSign($c)
	# 	}
	# 	grid $fmp.signLabel $fmp.signdist $fmp.signelev $fmp.signazi $fmp.signx $fmp.signy $fmp.signz

	grid configure $fmp.fromLabel $fmp.toLabel $fmp.deltaLabel -sticky "e"

	set fmc1 "$fm.control1"
	frame $fmc1
	button $fmc1.fromSet -text "from set" -command {::Gdhe::video::setPose ::Gdhe::video::poseFrom}
	button $fmc1.fromObs -text "from obs" -command {::Gdhe::video::setObs [array get ::Gdhe::video::poseFrom]}
	label $fmc1.separator -text "    "
	button $fmc1.toSet -text "to set" -command {::Gdhe::video::setPose ::Gdhe::video::poseTo}
	button $fmc1.toObs -text "to obs" -command {::Gdhe::video::setObs [array get ::Gdhe::video::poseTo]}
	grid $fmc1.fromSet $fmc1.fromObs $fmc1.separator $fmc1.toSet $fmc1.toObs

	set fmc2 "$fm.control2"
	frame $fmc2
	button $fmc2.go -text "Go" -command {::Gdhe::video::go 0}
	button $fmc2.goDump -text "Go & Dump" -command {::Gdhe::video::go 1}
	button $fmc2.stop -text "Stop" -command {::Gdhe::video::stop}	
	grid $fmc2.go $fmc2.goDump $fmc2.stop

	grid $fmp -padx 2 -pady 2
	grid $fmc1 -padx 2 -pady 2
	grid $fmc2 -padx 2 -pady 2 -sticky "e"

	#### dump ####

	set fd "$w.dump"
	labelframe $fd -text "dump"

	set fdf "$fd.filename"
	frame $fdf

	label $fdf.imageDirLabel -text "dir:"
	entry $fdf.imageDir -textvariable ::Gdhe::video::imageDir
	label $fdf.imagePrefixLabel -text "prefix:"
	entry $fdf.imagePrefix -textvariable ::Gdhe::video::imagePrefix
	label $fdf.imageFormatLabel -text "format:"
	tk_optionMenu $fdf.imageFormat ::Gdhe::video::imageFormat "eps" "ppm" "png"
	label $fdf.imageIndexLabel -text "index:"
	entry $fdf.imageIndex -width $::Gdhe::video::imageNbDigits -textvariable ::Gdhe::video::imageIndex

	grid $fdf.imageDirLabel $fdf.imageDir
	grid $fdf.imagePrefixLabel $fdf.imagePrefix
	grid $fdf.imageIndexLabel $fdf.imageIndex
	grid $fdf.imageFormatLabel $fdf.imageFormat

	grid configure $fdf.imageDirLabel $fdf.imagePrefixLabel $fdf.imageIndexLabel $fdf.imageFormatLabel -sticky "e"
	grid configure $fdf.imageIndex $fdf.imageFormat -sticky "w"

	grid $fdf -padx 2 -pady 2 -sticky "w"

	button $fd.dump -text "Dump" -command {::Gdhe::video::dump}

	grid $fd.dump -padx 2 -pady 2 -sticky "e"

	#### encode ####

	set fe "$w.encode"
	labelframe $fe -text "encode"

	set fef "$fe.filename"
	frame $fef

	label $fef.videoOutputLabel -text "output:" 
	entry $fef.videoOutput -textvariable ::Gdhe::video::videoOutput
	label $fef.videoCodecLabel -text "format:"
	tk_optionMenu $fef.videoCodec $::Gdhe::video::videoCodec "mpeg4"
	label $fef.videoFpsLabel -text "fps:"
	entry $fef.videoFps -width 5 -textvariable ::Gdhe::video::videoFps
	label $fef.videoBitrateLabel -text "bitrate:"
	entry $fef.videoBitrate -width 5 -textvariable ::Gdhe::video::videoBitrate

	grid $fef.videoOutputLabel $fef.videoOutput
	grid $fef.videoCodecLabel $fef.videoCodec
	grid $fef.videoFpsLabel $fef.videoFps
	grid $fef.videoBitrateLabel $fef.videoBitrate

	grid configure $fef.videoOutputLabel $fef.videoCodecLabel $fef.videoFpsLabel $fef.videoBitrateLabel -sticky "e"
	grid configure $fef.videoOutput $fef.videoCodec $fef.videoFps $fef.videoBitrate -sticky "w"

	grid $fef -padx 2 -pady 2 -sticky "e"

	button $fe.encode -text "Encode" -command {::Gdhe::video::encode}
	button $fe.play -text "Play" -command {::Gdhe::video::play}

	grid $fe.encode $fe.play -padx 2 -pady 2 -sticky "e"

	#### buttons ####

	set fc "$w.control"
	frame $fc

	button $fc.close -text "Close" -command "destroy $w"
	button $fc.help -text "Help" -command {::Gdhe::video::help}

	grid $fc.close $fc.help -padx 2 -pady 2

	grid $fm -sticky "we"
	grid $fd -sticky "we"
	grid $fe -sticky "we"
	grid $fc -sticky "e"

	grid configure $fm $fd $fe $fc -padx 2 -pady 2
    }

    proc setPose {poseArray} {
	set ${poseArray}(dist) $::obsDist
	set ${poseArray}(elev) $::obsElev
	set ${poseArray}(azi)  $::obsAzi
	set ${poseArray}(x)    $::obsX
	set ${poseArray}(y)    $::obsY
	set ${poseArray}(z)    $::obsZ
    }

    proc setObs {pose} {
	variable gdheOgl

	array set obsPose $pose

# 	$gdheOgl setObs \
# 	    -dist $obsPose(dist) \
# 	    -elev $obsPose(elev) \
# 	    -azi  $obsPose(azi)  \
# 	    -x    $obsPose(x)    \
# 	    -y    $obsPose(y)    \
# 	    -z    $obsPose(z)

	set ::obsDist $obsPose(dist)
	set ::obsElev $obsPose(elev)
	set ::obsAzi $obsPose(azi)
	set ::obsX $obsPose(x)
	set ::obsY $obsPose(y)
	set ::obsZ $obsPose(z)

	$gdheOgl redraw
    }

    proc go {{doDump 0}} {
	variable coord
	variable poseFrom 
	variable poseCur


	# compute translation signs
	foreach c $coord {
	    computePoseDeltaSignCoord $c
	}

	array set poseCur [array get poseFrom]

	nextTravellingPose $doDump
    }

    proc computePoseDeltaSignCoord {coord} {
	variable poseDeltaSign
	variable poseFrom
	variable poseTo
	set delta [expr $poseTo($coord) - $poseFrom($coord)]
	if {$delta != 0} {
	    set poseDeltaSign($coord) [sign $delta]
	}
    }

    proc nextTravellingPose {doDump} {
	variable coord
	variable taskId
	variable poseCur

	setObs [array get poseCur]
	if $doDump dump

	set hasNextPose 0
	
	foreach c $coord {
	    set hasNextPose [expr $hasNextPose || [nextTravellingPoseCoord $c]]
	}

	if $hasNextPose {
	    set taskId [after idle Gdhe::video::nextTravellingPose $doDump]
	} else {
	    setObs [array get poseCur]
	    if $doDump dump
	}
    }

    proc nextTravellingPoseCoord {coord} {
	variable poseCur
	variable poseTo
	variable poseDelta
	variable poseDeltaSign
	
	set delta [expr $poseTo($coord) - $poseCur($coord)]

	if {$delta == 0} {
	    return 0
	}

	if {[sign $delta] == $poseDeltaSign($coord)} {
	    set poseCur($coord) [expr $poseCur($coord) + $poseDeltaSign($coord)*$poseDelta($coord)]
	    return 1
	} else {
	    return 0
	}
    }

    proc sign {n} {
	if {$n == 0} {
	    return 1
	} else {
	    return [expr abs($n)/$n]
	}
    }

    proc nextTravellingPoseCoordElev {} {
    }

    proc nextTravellingPoseCoordAzi {} {
    }

    proc stop {} {
	variable taskId
	after cancel $taskId
    }

    proc dump {} {
	variable imageIndex
	variable imageFormat
	variable imageNbDigits
	variable imageTmp
	variable convertBin
	variable gdheOgl

	set index [format "%0${imageNbDigits}d" $imageIndex]
	switch $imageFormat {
	    "eps" {$gdheOgl dumpEps -color [getImageFilename $index]}
	    "ppm" {$gdheOgl dumpPpm -color [getImageFilename $index]}
	    "png" {
		$gdheOgl dumpPpm -color ${imageTmp}.ppm
		exec $convertBin ${imageTmp}.ppm [getImageFilename $index]
		exec rm -f ${imageTmp}.ppm
	    }
	}
	incr imageIndex
    }

    proc encode {} {
	variable mencoderBin
	variable videoOutput
	variable videoCodec
	variable videoFps
	variable videoBitrate

	switch $videoCodec {
	    "mpeg4" {
		set cmd "$mencoderBin mf://[getImageFilename] -mf fps=${videoFps} -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=${videoBitrate} -oac copy -o $videoOutput -ffourcc XVID"
		puts "\n$cmd \n"

		if {[catch "exec $cmd >@stdout 2>@stderr" result]} {
		    puts "$result"
		}
	    }
	}
    }

    proc play {} {
	variable mplayerBin
	variable videoOutput

	set cmd "$mplayerBin $videoOutput"
	puts "\n$cmd \n"

	if {[catch "exec $cmd >@stdout 2>@stderr" result]} {
	    puts "$result"
	}
    }

    proc getImageFilename {{index "*"}} {
	variable imageDir
	variable imagePrefix
	variable imageFormat
	
	return [file join $imageDir "${imagePrefix}${index}.${imageFormat}"]	
    }

    proc help {} {
	tk_messageBox -parent .video -title "Help" -type "ok" -icon "info" -message "Not yet available..."
    }
}

::Gdhe::video::init

